# Spotlight.js

Spotlight.js is a Javascript library that let developers highlight content on websites; it allows to create a tour, containing
several elements the user can browse through.

With Spotlight.js, you could:
* showcase your web app features;
* guide users wanting to accomplish a given task;
* attract users' attention on a given element.
* *do much more stuff!*

## Architecture

The library works the same way as a theatre set!

To perform a theatre play, you need actors, right? Well, same idea here! Each component you want to highlight can be 
declared as follows: 
```javascript
const titleActor = new Actor(() => document.getElementById('header-title'));
const menuActor = new Actor(() => document.getElementById('right-menu'));
```

Of course, you need a spotlight to showcase them!
```javascript
const spotlight = new Spotlight();
```

Then, add your actors as targets for your spotlight:
```javascript
spotlight.addActors( titleActor, menuActor );
```


## Play with it!

By default, `Spotlight` and `Actor` entities are injected in the global `window` object, allowing you to play with them 
in your browser console.

```javascript
const spotlight = new Spotlight({ options });       // see available options below
spotlight.addActors(
    new Actor(() => document.getElementById('paragraph-1'), { options }),
    new Actor(() => document.getElementById('paragraph-2'))
); 
```

## Options

You can set options:
* for your `Spotlight` instance;
* for your `Actors`, if you want to set settings for specific actors; these will override options set up in your `Spotlight` instance.

### `Spotlight` options

| name | description | default |
|---|---|---|
| `allowHiddenFocus` | allows Spotlight.js to focus elements even when the spot is not activated  | `false` |
| `autoScroll` | when an actor is not in the viewport (and spot is on), scrolls the document until the actor is visible | `true` |
| `centerActorOnFocus` | when an actor is focused, scrolls the view to center it, even if it is completely in the viewport | `true` | 
| `loops` | at the end of the tour, focuses on the first actor | `false` |
| `onBackgroundClick` | if `preventContentClicking` is true, set a callback to call when the user clicks the page during the tour | `(spotlight) => spotlight.toggleSpot()` |
| `onEnd` | callback triggered when calling `next()` while last actor is focused | `() => console.log("tour ended")` |
| `onLightsOff` | triggered when the spot goes off | `() => {}` | 
| `preventContentClicking` | prevents users from clicking page content when a presentation is in progress | `true` | 
| `preventFocusSpam` | blocks focus transition when another is running | `true` |
| `resetOnLightsOff` | focuses the first actor when turning the spot off | `false` |
| `resetOnEnd` | spot fades off when `next()` is called while last actor is focused | `true` |

### `Spotlight` + `Actor` options

| name | description | default |
|---|---|---|
| `delay` | transition time, in milliseconds, for the spot to switch from an actor to another | `1000` |
| `matchRadius` | reproduces the border-radius of the targeted actor | `true` |
| `onFocus` | callback triggered when the spot starts targeting a new actor | `(newActor: Actor, oldActor: Actor) => { console.log('switching focus from ${oldActor} to ${newActor}'); }` |
| `onFocused` | callback triggered when the spot targets a new actor | `(actor: Actor) => { console.log('now focusing ${actor}'); }` |
| `shadowColor` | color of the shadow (you might want to put some alpha value here) | `#ff0000dd` | 
| `showSign` | should display the sign | `true` |
| `showControls` | should display controls on the sign | `true` |
| `signHTML` | overrides sign body with custom HTML code | `''` |
| `signPosition` | defines position of the sign regarded the focused actor (can be `TOP`, `RIGHT`, `BOTTOM`, `LEFT`) | `SignPosition.RIGHT` |
| `signText` | text of the sign | `"Hello there!"` |
| `signTitle` | title of the sign | `"Title"` |

### Tips

* if you want to center the sign on the screen with no particular target (*eg* for beginning a tour), provide `() => null` as builder to the Actor instance :

``new Actor(() => null, {signTitle: 'Welcome to Spotlight.js', signText: 'Since no actor has been provided, the sign centers itself on the page by default.'})``