"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Spot_1 = require("./Spot");
const SizeWatcher_1 = require("./SizeWatcher");
const OptionBuilder_1 = require("../utils/options/OptionBuilder");
const Sign_1 = require("../sign/Sign");
class Spotlight {
    constructor(options) {
        this.targets = [];
        this.currentActorIndex = -1;
        this.options = OptionBuilder_1.OptionBuilder.buildOptions(options);
        this.isSwitching = false;
        this.spot = new Spot_1.default(this.options);
        this.sign = new Sign_1.Sign(this.options, () => this.previous(), () => this.next(), () => this.toggleSpot());
        Spotlight.instance = this;
        this.sizeWatcher = new SizeWatcher_1.default(() => __awaiter(this, void 0, void 0, function* () {
            console.log('Recomputing positions due to actor changes');
            const currentActor = this.targets[this.currentActorIndex];
            yield this.spot.adaptToActorSize();
            yield this.sign.forcePositionRecomputing(currentActor);
        }));
    }
    addActors(...actors) {
        for (const a of actors) {
            this.targets.push(a);
        }
    }
    toggleSpot() {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }
        this.isSwitching = true;
        const on = this.spot.isVisible;
        if (!on) {
            if (this.currentActorIndex === -1) {
                this.currentActorIndex = 0;
            }
            try {
                this._focusActor();
                this.spot.turnOn();
            }
            catch (error) {
                console.error(error);
            }
        }
        else {
            if (this.options.resetOnLightsOff) {
                this.currentActorIndex = -1;
            }
            this.spot.turnOff();
            this.sign.hide(true);
            this.sizeWatcher.disconnect();
            this.options.onLightsOff();
            setTimeout(() => this.isSwitching = false, this.options.delay);
        }
        return on;
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.spot.adaptToActorSize();
        });
    }
    next() {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }
        if (this.spot.isVisible || !this.spot.isVisible && this.options.allowHiddenFocus) {
            this.isSwitching = true;
            if (this.currentActorIndex < this.targets.length - 1 && this.currentActorIndex !== -1) {
                this.currentActorIndex++;
            }
            else if (this.currentActorIndex === this.targets.length - 1) {
                if (this.options.loops) {
                    this.currentActorIndex = 0;
                }
                else if (this.options.resetOnEnd) {
                    this.isSwitching = false;
                    this.toggleSpot();
                    this.currentActorIndex = 0;
                    this.options.onEnd();
                    return;
                }
            }
            else if (this.currentActorIndex === -1) {
                this.currentActorIndex =
                    this.targets.length > 1 ? 1 : 0;
            }
            this._focusActor();
        }
    }
    previous() {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }
        if (this.spot.isVisible || !this.spot.isVisible && this.options.allowHiddenFocus) {
            this.isSwitching = true;
            if (this.currentActorIndex > 0 && this.currentActorIndex !== -1) {
                this.currentActorIndex--;
            }
            else if (this.currentActorIndex === 0 && this.options.loops) {
                this.currentActorIndex = this.targets.length - 1;
            }
            else if (this.currentActorIndex === -1) {
                this.currentActorIndex = this.targets.length - 1;
            }
            this._focusActor();
        }
    }
    _focusActor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.targets.length === 0 || this.currentActorIndex === -1) {
                throw new Error('No actors have been registered, skipping.');
            }
            if (!this.options.allowHiddenFocus)
                this.spot.turnOn();
            this.sign.hide();
            const actor = this.targets[this.currentActorIndex];
            // listening to size changes
            this.sizeWatcher.disconnect();
            // launching sign content loading before showing it
            this.sign.setContent(actor);
            yield actor.build();
            this.sizeWatcher.observe(actor);
            this.spot.focus(actor)
                .then(() => __awaiter(this, void 0, void 0, function* () {
                yield this.update();
                this.isSwitching = false;
                yield this.sign.setHTMLMessage(actor);
            }));
        });
    }
}
exports.default = Spotlight;
